@@ -2,7 +2,7 @@
 #
 # Worksheet - A class for writing Excel Worksheets.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2016, John McNamara, jmcnamara@cpan.org
 #
 import re
 import datetime
@@ -36,7 +36,7 @@ def xl_rowcol_to_cell(row, col, row_abs=False, col_abs=False):
 
 def xl_rowcol_to_cell_fast(row, col):
     """
-    Optimised version of the xl_rowcol_to_cell function. Only used internally.
+    Optimized version of the xl_rowcol_to_cell function. Only used internally.
 
     Args:
        row: The cell row.    Int.
@@ -209,9 +209,55 @@ def xl_range_abs(first_row, first_col, last_row, last_col):
     return range1 + ':' + range2
 
 
+def xl_range_formula(sheetname, first_row, first_col, last_row, last_col):
+    """
+    Convert worksheet name and zero indexed row and col cell references to
+    a Sheet1!A1:B1 range formula string.
+
+    Args:
+       sheetname: The worksheet name.    String.
+       first_row: The first cell row.    Int.
+       first_col: The first cell column. Int.
+       last_row:  The last cell row.     Int.
+       last_col:  The last cell column.  Int.
+
+    Returns:
+        A1:B1 style range string.
+
+    """
+    cell_range = xl_range_abs(first_row, first_col, last_row, last_col)
+    sheetname = quote_sheetname(sheetname)
+
+    return sheetname + '!' + cell_range
+
+
+def quote_sheetname(sheetname):
+    """
+    Convert a worksheet name to a quoted  name if it contains spaces or
+    special characters.
+
+    Args:
+       sheetname: The worksheet name. String.
+
+    Returns:
+        A quoted worksheet string.
+
+    """
+
+    # TODO. Possibly extend this to quote sheetnames that look like ranges.
+    if not sheetname.isalnum() and not sheetname.startswith("'"):
+        # Double quote any single quotes.
+        sheetname = sheetname.replace("'", "''")
+
+        # Singe quote the sheet name.
+        sheetname = "'%s'" % sheetname
+
+    return sheetname
+
+
 def xl_color(color):
     # Used in conjunction with the XlsxWriter *color() methods to convert
-    # a colour name into an RGB formatted string. These colours are for
+    # a color name into an RGB formatted string. These colors are for
     # backward compatibility with older versions of Excel.
     named_colors = {
         'black': '#000000',
@@ -242,6 +288,16 @@ def xl_color(color):
     return "FF" + color.lstrip('#').upper()
 
 
+def get_rgb_color(color):
+    # Convert the user specified color to an RGB color.
+    rgb_color = xl_color(color)
+
+    # Remove leading FF from RGB color for charts.
+    rgb_color = re.sub(r'^FF', '', rgb_color)
+
+    return rgb_color
+
+
 def get_sparkline_style(style_id):
     styles = [
         {'series':   {'theme': "4", 'tint': "-0.499984740745262"},
@@ -549,10 +605,27 @@ def supported_datetime(dt):
     # Determine is an argument is a supported datetime object.
     return(isinstance(dt, (datetime.datetime,
                            datetime.date,
-                           datetime.time)))
+                           datetime.time,
+                           datetime.timedelta)))
 
 
-def datetime_to_excel_datetime(dt_obj, date_1904):
+def remove_datetime_timezone(dt_obj, remove_timezone):
+    # Excel doesn't support timezones in datetimes/times so we remove the
+    # tzinfo from the object if the user has specified that option in the
+    # constructor.
+    if remove_timezone:
+        dt_obj = dt_obj.replace(tzinfo=None)
+    else:
+        if dt_obj.tzinfo:
+            raise TypeError(
+                "Excel doesn't support timezones in datetimes. "
+                "Set the tzinfo in the datetime/time object to None or "
+                "use the 'remove_timezone' Workbook() option")
+
+    return dt_obj
+
+
+def datetime_to_excel_datetime(dt_obj, date_1904, remove_timezone):
     # Convert a datetime object to an Excel serial date and time. The integer
     # part of the number stores the number of days since the epoch and the
     # fractional part stores the percentage of the day.
@@ -567,16 +640,21 @@ def datetime_to_excel_datetime(dt_obj, date_1904):
     # We handle datetime .datetime, .date and .time objects but convert
     # them to datetime.datetime objects and process them in the same way.
     if isinstance(dt_obj, datetime.datetime):
-        pass
+        dt_obj = remove_datetime_timezone(dt_obj, remove_timezone)
+        delta = dt_obj - epoch
     elif isinstance(dt_obj, datetime.date):
         dt_obj = datetime.datetime.fromordinal(dt_obj.toordinal())
+        delta = dt_obj - epoch
     elif isinstance(dt_obj, datetime.time):
         dt_obj = datetime.datetime.combine(epoch, dt_obj)
+        dt_obj = remove_datetime_timezone(dt_obj, remove_timezone)
+        delta = dt_obj - epoch
+    elif isinstance(dt_obj, datetime.timedelta):
+        delta = dt_obj
     else:
         raise TypeError("Unknown or unsupported datetime type")
 
     # Convert a Python datetime.datetime value to an Excel date number.
-    delta = dt_obj - epoch
     excel_time = (delta.days
                   + (float(delta.seconds)
                      + float(delta.microseconds) / 1E6)
@@ -584,7 +662,8 @@ def datetime_to_excel_datetime(dt_obj, date_1904):
 
     # Special case for datetime where time only has been specified and
     # the default date of 1900-01-01 is used.
-    if dt_obj.isocalendar() == (1900, 1, 1):
+    if (not isinstance(dt_obj, datetime.timedelta)
+            and dt_obj.isocalendar() == (1900, 1, 1)):
         excel_time -= 1
 
     # Account for Excel erroneously treating 1900 as a leap year.
